
/*** block a  from ../../docs/pwg/advanced-scheduling.xml ***/
#include "filter.h"
#include <string.h>

static gboolean	gst_my_filter_activate	(GstPad      * pad);
static gboolean	gst_my_filter_activate_pull (GstPad  * pad,
					 gboolean      active);
static void	gst_my_filter_loop	(GstMyFilter * filter);

GST_BOILERPLATE (GstMyFilter, gst_my_filter, GstElement, GST_TYPE_ELEMENT);

/*** block b  from ../../docs/pwg/advanced-scheduling.xml ***/
static gboolean	gst_my_filter_setcaps	(GstPad  *pad,
					 GstCaps *caps);
static GstCaps *gst_my_filter_getcaps	(GstPad  *pad);

static void
gst_my_filter_base_init (gpointer klass)
{
  GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
  static GstElementDetails my_filter_details = {
    "An example plugin",
    "Example/FirstExample",
    "Shows the basic structure of a plugin",
    "your name <your.name@your.isp>"
  };
  static GstStaticPadTemplate sink_factory =
  GST_STATIC_PAD_TEMPLATE (
    "sink",
    GST_PAD_SINK,
    GST_PAD_ALWAYS,
    GST_STATIC_CAPS ("ANY")
  );
  static GstStaticPadTemplate src_factory =
  GST_STATIC_PAD_TEMPLATE (
    "src",
    GST_PAD_SRC,
    GST_PAD_ALWAYS,
    GST_STATIC_CAPS ("ANY")
  );

  gst_element_class_set_details (element_class, &my_filter_details);
  gst_element_class_add_pad_template (element_class,
        gst_static_pad_template_get (&src_factory));
  gst_element_class_add_pad_template (element_class,
        gst_static_pad_template_get (&sink_factory));
}

static void
gst_my_filter_class_init (GstMyFilterClass * klass)
{
}
static void
gst_my_filter_init (GstMyFilter * filter)
{

/*** block d  from ../../docs/pwg/advanced-scheduling.xml ***/
  GstElementClass *klass = GST_ELEMENT_GET_CLASS (filter);

  filter->sinkpad = gst_pad_new_from_template (
        gst_element_class_get_pad_template (klass, "sink"), "sink");
  gst_pad_set_setcaps_function (filter->sinkpad, gst_my_filter_setcaps);
  gst_pad_set_getcaps_function (filter->sinkpad, gst_my_filter_getcaps);

/*** block e  from ../../docs/pwg/advanced-scheduling.xml ***/
  gst_pad_set_activate_function (filter->sinkpad, gst_my_filter_activate);
  gst_pad_set_activatepull_function (filter->sinkpad,
      gst_my_filter_activate_pull);

/*** block f  from ../../docs/pwg/advanced-scheduling.xml ***/
  gst_element_add_pad (GST_ELEMENT (filter), filter->sinkpad);

  filter->srcpad = gst_pad_new_from_template (
        gst_element_class_get_pad_template (klass, "src"), "src");
  gst_element_add_pad (GST_ELEMENT (filter), filter->srcpad);

/*** block g  from ../../docs/pwg/advanced-scheduling.xml ***/
}

/*** block h  from ../../docs/pwg/advanced-scheduling.xml ***/
#include "caps.func"

/*** block i  from ../../docs/pwg/advanced-scheduling.xml ***/
static gboolean
gst_my_filter_activate (GstPad * pad)
{
  if (gst_pad_check_pull_range (pad)) {
    return gst_pad_activate_pull (pad, TRUE);
  } else {
    return FALSE;
  }
}

static gboolean
gst_my_filter_activate_pull (GstPad  *pad,
			     gboolean active)
{
  GstMyFilter *filter = GST_MY_FILTER (GST_OBJECT_PARENT (pad));

  if (active) {
    filter->offset = 0;
    return gst_pad_start_task (pad,
        (GstTaskFunction) gst_my_filter_loop, filter);
  } else {
    return gst_pad_stop_task (pad);
  }
}

/*** block j  from ../../docs/pwg/advanced-scheduling.xml ***/
#define BLOCKSIZE 2048

static void
gst_my_filter_loop (GstMyFilter * filter)
{
  guint64 len;
  GstFormat fmt = GST_FORMAT_BYTES;
  GstBuffer *buf = NULL;

  if (!gst_pad_query_position (filter->sinkpad, &fmt, NULL, &len)) {
    goto stop;
  } else if (filter->offset >= len) {
    gst_pad_push_event (filter->sinkpad, gst_event_new (GST_EVENT_EOS));
  } else if (gst_pad_pull_range (filter->sinkpad, filter->offset,
				 BLOCKSIZE, &buf) != GST_FLOW_OK ||
	     gst_pad_push (filter->sinkpad, buf) != GST_FLOW_OK) {
    goto stop;
  } else {
    filter->offset += BLOCKSIZE;
    return;
  }

stop:
  gst_pad_pause_task (filter->sinkpad);
}

/*** block k  from ../../docs/pwg/advanced-scheduling.xml ***/
#include "register.func"
